//
//  SelectFolder.cpp  -- functions for accessing folders and retrieving
//    message headers
//  -- created 4/20/00  updated 4/20/00
//////////////////////////////////////////////

#include "SelectFolder.h"

extern Althea gAlthea;

string readTilEOL(int sock, int &errnum)  {
   string buffer;

   /* try to check for errors in an appropriate order.
      check for more specific matches prior to the less
      specific.
   */

   buffer = readTilEOL(sock);
   if (gAlthea.get_Verbose()==2)
     cout << "S: " << buffer << endl;

   if (strstr(buffer.c_str(), "* BYE ")!=NULL)  {
      // lost connection to server.
      // must re-login before anything else
      // can be done with the network again.
      errnum = LOST_CONNECTION_TO_SERVER;
      return "";
   }
   else if ((strstr(buffer.c_str()," NO ")!=NULL) &&
            (strstr(buffer.c_str()," lock ")!=NULL))  {
      // lost lock on folder
      errnum = SUCCESS;   
      return(buffer);
   }
   else if ((strstr(buffer.c_str(), " RECENT")!=NULL) &&
            (strstr(buffer.c_str(), "* ")!=NULL) &&
	    (strstr(buffer.c_str(), " 0 ")==NULL))  {
      // new mail in this folder
      errnum = NEW_MAIL;
      return(buffer);
   }
   else if (strstr(buffer.c_str()," NO ")!=NULL)  {
     // command failed
      errnum = COMMAND_TO_SERVER_FAILED;
      return("");
   }
   else if (strstr(buffer.c_str()," BAD ")!=NULL)  {
      // command unknown or arguments invalid
      errnum = COMMAND_OR_ARGUMENTS_INVALID;
      return("");
   }
   else  {
      errnum = SUCCESSFUL_SERVER_READ;
      return(buffer);
   }

}

void EraseBefore(string &haystack, const string &needle)  {
   haystack.erase(0,haystack.find_first_of(needle)+1);
}

void GetLineText(const string &haystack, const string &keyword, string &value)  {
   // defs
   unsigned int afterKeyword;
   value = "";

   // figure it out
   if (strstr(haystack.c_str(),keyword.c_str())!=NULL)  {
      afterKeyword = haystack.find(keyword) + keyword.length();
      for (; haystack[afterKeyword]!='\n' && afterKeyword<=haystack.length(); 
             afterKeyword++)
         value += haystack[afterKeyword];
   }
   else
      value = "";
}

int SelectFolder(int sock, const string &foldername, int &numMessages, 
		 string &folderOpenStatus)  {
   // defs
   string command;
   string response;
   string tmpString;
   int startEXISTS;
   int begStatus;
   int endStatus;
   int lengthStatus;
   int errnum=SUCCESS;

   // make command
   command = "a003 SELECT \"" + foldername + "\"\r\n";

   // send command to server
   write(sock, command.c_str(), strlen(command.c_str()));

   // read response
   do {
      response = readTilEOL(sock,errnum);
      // right now we only really care about 
      // how many messages are in the mailbox 
      if (strstr(response.c_str()," EXISTS"))  {
	 // we know that the message is in the form
	 // * <SPACE> int <SPACE> EXISTS
	 // -- must find out where that int ends,
	 // which is posIndex for the start of 
	 // exists - 1
	 startEXISTS = response.find_first_of(" EXISTS");

	 // find the int substring and convert it
	 // to type int
         tmpString = response.substr(2,startEXISTS-3);
         numMessages = atoi(tmpString.c_str());
      }
   }  while ((!strstr(response.c_str(),"a003 OK ")) && (errnum<END_SUCCESS)); 

   // the final response told us our read/write
   // status for this folder
   // or if the folder open was unsuccessful, some
   // info we don't want.  check it first
   if (errnum<END_SUCCESS)  {
      begStatus = response.find("[")+1;
      endStatus = response.find("]");
      lengthStatus = endStatus-begStatus;
      folderOpenStatus = response.substr(begStatus,lengthStatus);
   }
   else  {
      folderOpenStatus = "Open Failed";
   }

   return(errnum);
}

string GetFlags(int sock,int msgNumber, int &errnum)  {
   // defs
   string response;
   string flags;
   string command;
   string msgStringCounter;

   // build the command
   msgStringCounter = itoa(msgNumber);
   command = "a" + msgStringCounter + " FETCH "
          + msgStringCounter + " FLAGS\r\n";

   // send the command out
   write(sock,command.c_str(),strlen(command.c_str()));

   // read the response
   response = readTilEOL(sock,errnum);
   if (errnum>BEG_FAIL)
      return("");

   // remove FLAGS parens
   EraseBefore(response,"("); 
   EraseBefore(response,"("); 
   flags = response.substr(0,response.find(")"));

   // get rid of the line saying
   // amy extra lines
   while (!strstr(response.c_str(),"OK FETCH completed"))  {
      response = readTilEOL(sock,errnum);
      if (errnum>BEG_FAIL)
         return("");
   }

   // end of function
   // return success
   return(flags);
}

int GetMessageHeader (int sock, int msgNumber, string &to, 
		      string &from, string &subject, int &uid,
		      string & date)  {
      // defs
      string msgStringCounter;
      string command;
      string header;
      string response;
      string tmpHeader;
      string tmpLength;
      int length;
      int end;
      int beg;
      int headerCounter;
      int i;
      int errnum=SUCCESS;
      char c;

      // compose our command
      msgStringCounter = itoa(msgNumber);
      command = "a" + msgStringCounter + " FETCH "
	            + msgStringCounter + " BODY.PEEK[header]\r\n";
      
      /********************************************
       * get all of our header information from   *
       * the Fetch BODY[header] command           *
       * ******************************************/

      // send it out and
      // find the size of our header
      write(sock,command.c_str(),strlen(command.c_str()));
      response = readTilEOL(sock,errnum);
      if (errnum>BEG_FAIL)
         return(errnum);

      // the first line coming back will
      // have the length of the rest of our info
      beg = response.find("{") + 1;
      end = response.find("}");
      tmpLength = response.substr(beg,end-beg);
      length = atoi(tmpLength.c_str());

      // start reading in the rest of the header
      header = "";
      i = 1;
      for (headerCounter=0; headerCounter<=length && i>0; headerCounter++)  {
         i = ReadFromSocket(sock,&c,1);
	 header += c;
      }
      // get rid of the line saying
      // amy extra lines
      while (!strstr(response.c_str(),"OK FETCH completed"))  {
         response = readTilEOL(sock,errnum);
	 if (errnum>BEG_FAIL) 
	    return(errnum);
      }
      
      // get the date
      GetLineText(header,"Date:",date);

      // get sender (from)
      GetLineText(header,"From:",from);

      // get subject
      GetLineText(header,"Subject:",subject);
      
//      // get UID
//      GetLineText(header,"Message-Id:",uid);    this is not the UID
      
      // get who it is to
      GetLineText(header,"To:",to);


// get UID
      errnum=IMAPSeq2UID(sock, msgNumber, uid);

      return(errnum);
}

int GetMessageHeaders(int sock,Folder &folder)  {
   // defs
   int numMessages=0;
   int errnum=SUCCESS;
   int msgCounter;
   string foldername = folder.get_Full_Folder_Name();
   string folderOpenStatus;
   string command;
   string header;
   string flags;
   string to;
   string from;
   string subject;
   string date;
   int uid;
   Message message;

   // select a folder
   errnum = SelectFolder(sock,foldername,numMessages,folderOpenStatus);
   if (errnum>BEG_FAIL)
      return(errnum);

   // loop through and start picking up message
   // headers, adding a new message instance to
   // our folder with each iteration

   if (gAlthea.get_Verbose() == 1)
                cout << endl;


   for (msgCounter=1; msgCounter<=numMessages; msgCounter++)  {
      // get the headers
      errnum = GetMessageHeader(sock,msgCounter,to,from,subject,uid,date);
      if (errnum>BEG_FAIL)
         return(errnum);

      // get the flags for this message
      flags = GetFlags(sock,msgCounter,errnum);
      if (errnum>BEG_FAIL)
         return(errnum);
      if (gAlthea.get_Verbose()==1)
        cout << "Got headers for message " << msgCounter << " of " << numMessages << ".\r";
      // insert the stuff into out message
      message.set_From_Address(from);
      // do we care about this one?
      //message.set_To_Adress(to);
      message.set_Subject(subject);
      message.set_To_Address(to);
      message.parse_Date( date );
      message.set_Have_Body( 0 );
      
      // this is not the uid here.
      // it is the relative msg number/sequence id
      message.set_Message_ID(msgCounter);
      message.set_UID(uid);
      message.set_Sequence_Number(msgCounter);
      message.set_Server_Ptr( folder.get_Server_Ptr() );
  
    


      
      //lets see if we need to filter the suckah
   if (flags.find("een") > flags.length() && flags.find("eleted")> flags.length() && folder.get_Folder_Name()=="INBOX") 
     {
       if (gAlthea.get_Verbose())
	 cout << "I could filter it..." << endl;
       list<Filter>::iterator it;
       	it = gAlthea.filter_list.begin();
       	while( it != gAlthea.filter_list.end() )
       	  {
       	    if ((((*it).field == "from") && (from.find((*it).filter_string)<=from.length())) ||
		(((*it).field == "to") && (to.find((*it).filter_string)<=to.length())) ||
		(((*it).field == "subject") && (subject.find((*it).filter_string)<=subject.length()))) 
       	      {
	
		if ((*it).folder != "") 
		  {
       		    IMAPCopyMessage(sock, uid, (*it).folder, folder.get_Server_Ptr()->get_Folder_Root());
		  }
		cout << "filtered" << endl;
		message.Delete();
		  }
       	    it++;
       	  }
	flags="";
	IMAPSetFlagByUID(sock, uid, "", flags);
	if (gAlthea.get_Verbose())
	  cout << flags << endl;
     }

   message.set_Flags( flags );
   folder.add_Message(message);
   }
   
   






   // anything we didn't pick up?

   // no errors
   // return success
   if (gAlthea.get_Verbose()==1)
     cout << "\n";
   return(SUCCESS);
}

int GetNewMail(int sock, Folder &folder)  {
   // defs
   Message message;
   string command;
   string response;
   string tmpString;
   string to,from,subject,date;
   int uid;
   string flags;
   int startEXISTS;
   int highNumMessages;
   int errnum=SUCCESS;
   int msgCounter;

   // figure out how many messages are in the folder now
   command = "a023 EXAMINE " + folder.get_Folder_Name() + "\r\n";
   write(sock,command.c_str(),strlen(command.c_str()));

   // read response
   do {
      response = readTilEOL(sock,errnum);
      // right now we only really care about 
      // how many messages are in the mailbox 
      if (strstr(response.c_str()," EXISTS"))  {
	 // we know that the message is in the form
	 // * <SPACE> int <SPACE> EXISTS
	 // -- must find out where that int ends,
	 // which is posIndex for the start of 
	 // exists - 1
	 startEXISTS = response.find_first_of(" EXISTS");

	 // find the int substring and convert it
	 // to type int
         tmpString = response.substr(2,startEXISTS-3);
         highNumMessages = atoi(tmpString.c_str());
      }
   }  while ((!strstr(response.c_str(),"a023 OK ")) && (errnum<END_SUCCESS)); 

   // loop through and start picking up message
   // headers, adding a new message instance to
   // our folder with each iteration
   for (msgCounter=folder.get_Sequence_Stop()+1; msgCounter<=highNumMessages; 
                msgCounter++)  {
      // get the headers
      errnum = GetMessageHeader(sock,msgCounter,to,from,subject,uid,date);
      if (errnum>BEG_FAIL)
         return(errnum);

      // get the flags for this message
      flags = GetFlags(sock,msgCounter,errnum);
      if (errnum>BEG_FAIL)
         return(errnum);
      
      // insert the stuff into out message
      message.set_From_Address(from);
      // do we care about this one?
      //message.set_To_Adress(to);
      message.set_Subject(subject);
      message.set_To_Address(to);
      message.parse_Date( date );
      message.set_Flags( flags );
      message.set_Have_Body( 0 );
      
      // this is not the uid here.
      // it is the relative msg number/sequence id
      message.set_Message_ID(msgCounter);
      message.set_UID(uid);
      message.set_Sequence_Number(msgCounter);
      message.set_Server_Ptr( folder.get_Server_Ptr() );



     
      
      
      // add the message to our folder
      folder.add_Message(message);
   }
   
   return(errnum);
}













