//
//  Server.h  --  Server class definition
//  -- created  4/13/00  updated 4/19/00
//////////////////////////////////////////////

#ifndef SERVER_H
#define SERVER_H


#include <string>
#include <list>
#include "Folder.h"

#define IMAPPORTNO 143

enum server_state 
    {
        NONAUTH, AUTH, SELECTED, LOGOUT 
    };


class Server
{
private:
    string server_name;
    string smtp_server_name;
    string imap_server_name;
    int port_number;
    string username;
    string password;
    string folder_root;
    string fccfolder;
    list<Folder> folder_list;
    server_state current_state;
    int network_socket;
    
public:

    // Constructors
    Server()
    {
	Server( "", "", "", IMAPPORTNO, "", "", "", "" );
    };

    Server( string s_name, string smtp_name, string imap_name, int p_num, string uname, string pass, string f_root, string fcc_f )
    {
	server_name = s_name;
        smtp_server_name = smtp_name;
        imap_server_name = imap_name;
	port_number = p_num;  
	username = uname;
	password = pass;
        folder_root = f_root;
	fccfolder= fcc_f;
    };

    // Mutators
    void set_Server_Name( const string &);
    void set_Port_Number( int );
    void set_Username( const string &);
    void set_Password( const string &);
    void set_Current_State( server_state );
    
    // Accessors
    string get_Server_Name();
    string get_SMTP_Server_Name();
    string get_IMAP_Server_Name();
    int get_Port_Number();
    string get_Username();
    string get_Password();  // This has limited uses, may be discarded later
    string get_FCC_Folder();
    int get_Num_Folders();
    server_state get_Current_State();
    Folder * get_Folder( int );
    int get_Socket();
    string get_Folder_Root();
    
    // Connection-related Functions
    int login();
    int logout();
    int check_Connected();
    int get_Folder_List_From_Server();

    // Folder-related Functions
    void add_Folder( Folder &);
    void delete_Folder( const string &name);         // delete based on folder name
    int get_New_Mail();
    
    // Debugging purposes only
    void display();
        

    
};

#endif
