//
//  imap.cpp  --  IMAP functions 
//  -- created 5/23/00 updated 5/23/00
///////////////////////////////////////////

#include	"imap.h"

extern Althea gAlthea;

//IMAPLogin logs into the host, returns 0 if successful or 102 if not

int IMAPLogin(int &sock, const string &host, const int port, const string
&username, const string &password)
{

string	errcode;
int err;


	/* connect to port "port" on "host" */
        sock = MakeConnection( host.c_str(), port );

	errcode = readTilEOL(sock);
	if (strncmp(errcode.c_str(),gIMAPStarOK.c_str(),strlen(gIMAPStarOK.c_str()))!=0) 
	{
		return SERVER_NOT_READY_FOR_CONNECTION;
	}
	
	// make login command 

	string logincommand=gIMAPLoginTag + " " + gIMAPLoginCommand + " \"";
	logincommand += username;
	logincommand += gIMAPQuoteSpaceQuote;
	logincommand += password;
	logincommand += gIMAPQuoteEOL;
	
	// make noop command
	string noopcommand=gIMAPCheckMailTag + " " + gIMAPCheckMailCommand;
	noopcommand += gIMAPEOL;

	//Send logincommand


        write(sock, logincommand.c_str() , strlen(logincommand.c_str()));
	if (gAlthea.get_Verbose()==2)
	  cout << "C: " << logincommand << endl;
	  
	string OKResponse=gIMAPLoginTag + " " + gIMAPOK;

	// read response if not what we expect, return error code 102
	errcode = readTilEOLTilExpected(sock, gIMAPLoginTag, gIMAPLoginExpected, err);
	if (strncmp(errcode.c_str(),OKResponse.c_str(),strlen(OKResponse.c_str()))!=0) 
	{
		return BAD_LOGIN;
	}
	return SUCCESS;
}



// Get folder listing

int getIMAPFolders(int sock, list<string>& folders, string folderroot)
{
string line,foldername;
int i;


// check for INBOX

	// form command
	string isINBOXcommand = gIMAPListTag + " " + gIMAPListCommand + " " + gIMAPQuoteQuote + " \"" + gIMAPINBOXName + gIMAPQuoteEOL;

	// send command
        write(sock, isINBOXcommand.c_str(), strlen(isINBOXcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << isINBOXcommand << endl;
	// read in response
	line = readTilEOL(sock);

	// if it is not a list item, this server must not have an "INBOX" otherwise (if it does give an item,) it does....
	if (strncmp(line.c_str(),gIMAPListResponse.c_str(),strlen(gIMAPListResponse.c_str()))==0)
	{
		folders.push_back(gIMAPINBOXName);        // add INBOX to the list of folders
		line = readTilEOL(sock); // to get rid of "OK" line
	}


// Check for everything else


	// form command
	string listcommand = gIMAPListTag + " " + gIMAPListCommand + " " + gIMAPQuoteQuote + " \"" + 
		folderroot + "/" + gIMAPListWildcard + gIMAPQuoteEOL;


	// Send command
        write(sock, listcommand.c_str() , strlen(listcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << listcommand << endl;

	// Get first response	
	line = readTilEOL(sock);


	// while it is giving us list items, process them
	while (strncmp(line.c_str(),gIMAPListResponse.c_str(),strlen(gIMAPListResponse.c_str()))==0) 
	{
		i=0;
		//go until you pass the flags, if we ever do sub folders, we will need to
		//parse those too
			int lastslash=0;
			while (line.c_str()[i] !=0)
			{
				if (line.c_str()[i]=='/') { lastslash=i; }
				if (line.c_str()[i]=='\"' && line.c_str()[i+1]!=0)
                                {
                                lastslash=i;
                                }
				i++;
			}


			i=lastslash+1;
		if (line.c_str()[i]!='.') 
		{
			foldername="";
			while (line.c_str()[i] !=0 && line.c_str()[i] != '\"' && line.c_str()[i]!='\r')
			{

				foldername+=line.c_str()[i];
				i++;
			}
			if (foldername!="")
			  folders.push_back(foldername); // add the folder to the list
		}
	line = readTilEOL(sock); // read the next line
	}

	// if this isn't what we expect, there might have been a server error. 
	// However, even if we didn't ask for a valid directory it won't complain here.
	
	string commandDoneTag = gIMAPListTag + " " + gIMAPOK;

	if (strncmp(line.c_str(),commandDoneTag.c_str(),strlen(commandDoneTag.c_str()))==0) 
	{
		return SUCCESS;
	} else {
		return LIST_FAILED;
	}

}

// Get folder listing

int IMAPExpunge(int sock, list<int>& deleted)
{
string line,foldername,message;
int i,mess,err;


	// form command
	string expungecommand = gIMAPExpungeTag + " " + gIMAPExpungeCommand + gIMAPEOL;

	// send command
        write(sock, expungecommand.c_str(), strlen(expungecommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << expungecommand << endl;
	// read in response

	line = readTilEOLTilExpected(sock,gIMAPExpungeExpected,gIMAPExpungeTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}
	// while it is giving us list items, process them
	while (strncmp(line.c_str(),gIMAPItemMarker.c_str(),strlen(gIMAPItemMarker.c_str()))==0) 
	{
		i=gIMAPExpungeCharsToItem;
		if (line.c_str()[i]!='.') 
		{
			message="";
			while (line.c_str()[i] !=' ')
			{
				message+=line.c_str()[i];
				i++;
			}
			mess = atoi(message.c_str());

			deleted.push_back(mess); // add the folder to the list
		}
	line = readTilEOLTilExpected(sock,gIMAPExpungeExpected,gIMAPExpungeTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}


	}

	// if this isn't what we expect, there might have been a server error. 
	// However, even if we didn't ask for a valid directory it won't complain here.
	return err;
}



// get the message body by the UID (this is a different IMAP command than 
// getting the message by the index, so if we do that it will need to be 
// a different command


int IMAPGetMessageBodyByUID(int sock, int UID, string &messagetext)
{

string line;
int size=0;
char c;
int z=0;
int err;

	// form command
	string fetchcommand = gIMAPFetchTag + " " + gIMAPFetchCommand + " " + itoa(UID) + " " + gIMAPBodyMIMEPart + gIMAPEOL;
	// send command
        write(sock, fetchcommand.c_str() , strlen(fetchcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << fetchcommand << endl;
	// read response
	line = readTilEOLTilExpected(sock,gIMAPFetchExpected,gIMAPFetchTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}
	if (strncmp(gIMAPItemMarker.c_str(),line.c_str(),strlen(gIMAPItemMarker.c_str()))==0) //check that the message actually exists
	{

		int i=0;
		//go until you pass the flags, if we ever do sub folders, we will need to
		//parse those too.
		while (line.c_str()[i] != '{' && line.c_str()[i] != '\"')
		{
			i++;
		}
		if (line.c_str()[i] == '\"')
		{
			messagetext="";
			line = readTilEOLTilExpected(sock, gIMAPFetchExpected, gIMAPFetchTag, err);
			return err;
		}
		// pass over the {
		i++;


		// use temp to store the number of bytes as a string
		string temp;
		while (line.c_str()[i] != '}')
		{
			temp += line.c_str()[i];
			i++;
		}

		// convert the string to an int
		size=atoi(temp.c_str());
		messagetext="";
		// read size bytes in from the network and put it on message text
	        i = ReadFromSocket( sock, &c, 1 );
       		while( i > 0  && z<size )
        	{
			messagetext+=c;
               		i = ReadFromSocket( sock, &c, 1 );
			z++;
        	}
	line = readTilEOLTilExpected(sock, gIMAPFetchTag, gIMAPFetchTag, err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}


			
	} else {
		return MESSAGE_NOT_THERE; //message doesn't exist
	}

return err;

}


// get the message header by the UID (this is a different IMAP command than 
// getting the message by the index, so if we do that it will need to be 
// a different command


int IMAPGetMessageHeaderByUID(int sock, int UID, string &messagetext)
{

string line;
int size=0;
char c;
int z=0;
int err;
string sUID = itoa(UID);

	// form command
	string fetchcommand = gIMAPFetchHeaderTag + " " + gIMAPFetchCommand + " " + itoa(UID) + " " + gIMAPHeaderMIMEPart + gIMAPEOL;
	// send command
        write(sock, fetchcommand.c_str() , strlen(fetchcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << fetchcommand << endl;
	// read response
	line = readTilEOLTilExpected(sock,gIMAPFetchHeaderTag,gIMAPFetchExpected,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}
	

	if (strncmp(gIMAPItemMarker.c_str(),line.c_str(),strlen(gIMAPItemMarker.c_str()))==0) //check that the message actually exists
	{

		int i=0;
		//go until you pass the flags, if we ever do sub folders, we will need to
		//parse those too.
		while (line.c_str()[i] != '{')
		{
			i++;
		}
		// pass over the {
		i++;


		// use temp to store the number of bytes as a string
		string temp;
		while (line.c_str()[i] != '}')
		{
			temp += line.c_str()[i];
			i++;
		}

		// convert the string to an int
		size=atoi(temp.c_str());
		messagetext="";
		// read size bytes in from the network and put it on message text
	        i = ReadFromSocket( sock, &c, 1 );
       		while( i > 0  && z<size )
        	{
			messagetext+=c;
               		i = ReadFromSocket( sock, &c, 1 );
			z++;
        	}

	line = readTilEOLTilExpected(sock,gIMAPFetchHeaderTag,gIMAPFetchHeaderTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}


			
	} else {
		return MESSAGE_NOT_THERE; //message doesn't exist
	}

return err;

}

// get the Set flags by the UID (this is a different IMAP command than 
// getting the message by the index, so if we do that it will need to be 
// a different command


int IMAPSetFlagByUID(int sock, int UID, const string &flagtoset, string
&currentflags)
{

string line;
int err;



	// form command
	string fetchcommand = gIMAPSetFlagTag + " " + gIMAPStoreCommand + " " + itoa(UID) + " " + gIMAPSetFlagCommand + " (" + flagtoset + ")" + gIMAPEOL; 

	
	// send command
        write(sock, fetchcommand.c_str() , strlen(fetchcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << fetchcommand << endl;
	// read response

	line = readTilEOLTilExpected(sock, gIMAPFetchExpected, gIMAPSetFlagTag, err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strncmp(gIMAPItemMarker.c_str(),line.c_str(),strlen(gIMAPItemMarker.c_str()))==0) //check that the message actually exists
	{

		int i=0;
		//go until you pass the message id
		while (line.c_str()[i] != '(')
		{
			i++;
		}
		i++;
		//go until you pass SOMETHING ELSE
		while (line.c_str()[i] != '(')
		{
			i++;
		}
		// pass over the {
		i++;


		// use temp to store the number of bytes as a string
		string temp;
		while (line.c_str()[i] != ')')
		{
			currentflags += line.c_str()[i];
			i++;
		}

		line = readTilEOLTilExpected(sock,gIMAPSetFlagTag,gIMAPSetFlagTag,err);
		if (err==LOST_CONNECTION_TO_SERVER) {return err;}



			
	} else {
		return MESSAGE_NOT_THERE; //message doesn't exist
	}

return err;

}
// get the unSet flags by the UID (this is a different IMAP command than 
// getting the message by the index, so if we do that it will need to be 
// a different command


int IMAPUnSetFlagByUID(int sock, int UID, const string &flagtounset, string
&currentflags)
{

string line;
int err;



	// form command
	string fetchcommand = gIMAPSetFlagTag + " " + gIMAPStoreCommand + " " + itoa(UID) + " " + gIMAPUnSetFlagCommand + " (" + flagtounset + ")" + gIMAPEOL; 

	
	// send command
        write(sock, fetchcommand.c_str() , strlen(fetchcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << fetchcommand << endl;
	// read response

	line = readTilEOLTilExpected(sock, gIMAPFetchExpected, gIMAPSetFlagTag, err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strncmp(gIMAPItemMarker.c_str(),line.c_str(),strlen(gIMAPItemMarker.c_str()))==0) //check that the message actually exists
	{

		int i=0;
		//go until you pass the message id
		while (line.c_str()[i] != '(')
		{
			i++;
		}
		i++;
		//go until you pass SOMETHING ELSE
		while (line.c_str()[i] != '(')
		{
			i++;
		}
		// pass over the {
		i++;


		// use temp to store the number of bytes as a string
		string temp;
		while (line.c_str()[i] != ')')
		{
			currentflags += line.c_str()[i];
			i++;
		}

		line = readTilEOLTilExpected(sock,gIMAPSetFlagTag,gIMAPSetFlagTag,err);
		if (err==LOST_CONNECTION_TO_SERVER) {return err;}



			
	} else {
		return MESSAGE_NOT_THERE; //message doesn't exist
	}

return err;

}

//
// Create a folder
// 


int IMAPCreateFolder(int sock, const string &foldername, const string &folderroot)
{

string line;
int err;


	// form command

	string createcommand = gIMAPCreateFolderTag + " " + gIMAPCreateFolderCommand + " " + folderroot + gIMAPDirectoryIndicator + foldername + gIMAPEOL;

	
	// send command
        write(sock, createcommand.c_str() , strlen(createcommand.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << createcommand << endl;
	// read response

	line = readTilEOLTilExpected(sock, gIMAPCreateFolderExpected, gIMAPCreateFolderTag, err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strstr(line.c_str(),gIMAPCreateFolderSuccessful.c_str())!=NULL) {
		return err;		
	} else if (strstr(line.c_str(),gIMAPCreateFolderFailed.c_str())!=NULL) { 
		return CREATE_FOLDER_FAILED;
	} else	{
		return CREATE_FOLDER_FAILED;
	}

}


//
// Delete a folder
// 


int IMAPDeleteFolder(int sock, const string &foldername, const string &folderroot)
{

string line;
int err;
if (foldername != gIMAPINBOXName)
{
	// form command
	string command = gIMAPDeleteFolderTag + " " + gIMAPDeleteFolderCommand + " " + folderroot + gIMAPDirectoryIndicator + foldername + gIMAPEOL;

	
	// send command
        write(sock, command.c_str() , strlen(command.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << command << endl;
	// read response

	line = readTilEOLTilExpected(sock,gIMAPDeleteFolderExpected,gIMAPDeleteFolderTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strstr(line.c_str(), gIMAPDeleteFolderSuccessful.c_str())!=NULL) {
		return err;		
	} else if (strstr(line.c_str(),gIMAPDeleteFolderFailedDueToInferior.c_str())!=NULL) { 
		return DELETE_FAILED_DUE_TO_INFERIOR;
	} else if (strstr(line.c_str(),gIMAPDeleteFolderFailed.c_str())!=NULL){
		return DELETE_FAILED;
	} else {
		return DELETE_FAILED;
	}
} else {
return NO_DELETE_INBOX;
}



}


//
// Rename a folder
// 


int IMAPRenameFolder(int sock, const string &fromname, const string &toname, const string &folderroot)
{
int err;
string line;
string command="";
	if (fromname==gIMAPINBOXName) {
		// form command
		command = gIMAPRenameFolderTag + " " + gIMAPRenameFolderCommand + " " + fromname + " " + folderroot + gIMAPDirectoryIndicator + toname + gIMAPEOL;
	} else {
		// form command
		command = gIMAPRenameFolderTag + " " + gIMAPRenameFolderCommand + " " + folderroot + gIMAPDirectoryIndicator + fromname + " " + folderroot + gIMAPDirectoryIndicator + toname + gIMAPEOL;
	}
	
	// send command
        write(sock, command.c_str() , strlen(command.c_str()));
        if (gAlthea.get_Verbose()==2)
          cout << "C: " << command << endl;	
	// read response

	line = readTilEOLTilExpected(sock,gIMAPRenameFolderExpected,gIMAPRenameFolderTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strstr(line.c_str(),gIMAPRenameFolderSuccessful.c_str())!=NULL) {
		return err;		
	} else if (strstr(line.c_str(),gIMAPRenameFolderFailed.c_str())!=NULL){
		return RENAME_FAILED;
	} else {
		return RENAME_FAILED;
	}

}



int IMAPLogout(int sock)
{
	// form command
	string logoutcommand = gIMAPLogoutTag + " " + gIMAPLogoutCommand + gIMAPEOL;
	// send command
        write(sock, logoutcommand.c_str() , strlen(logoutcommand.c_str()));
        if (gAlthea.get_Verbose()==2)
          cout << "C: " << logoutcommand << endl;
	return SUCCESS;
}



int IMAPSeq2UID(const int sock, const int seq, int &UID)
{
string line;
string command="";
int i,err;

	// form command
	command = gIMAPSeq2UIDTag + " " + gIMAPFetchUIDCommand + " " +
itoa(seq) + " " + gIMAPUIDMIMEPart + gIMAPEOL; 
	
	// send command
        write(sock, command.c_str() , strlen(command.c_str()));
        if (gAlthea.get_Verbose()==2)
          cout << "C: " << command << endl;	
	// read response
	line = readTilEOLTilExpected(sock,gIMAPFetchExpected,gIMAPSeq2UIDTag,err);
		if (err==LOST_CONNECTION_TO_SERVER) {return err;}
	if (strstr(line.c_str(),"*")!=NULL) {

		i=2;
		// use temp to store the number of bytes as a string
		while (line.c_str()[i] != '(')
		{
			i++;
		}
		i=i+5;
		string temp="";

		while (line.c_str()[i] != ')')
		{
			temp += line.c_str()[i];
			i++;
		}
		UID=atoi(temp.c_str());
		line = readTilEOLTilExpected(sock,gIMAPSeq2UIDTag,gIMAPSeq2UIDTag,err);
		if (err==LOST_CONNECTION_TO_SERVER) {return err;}
		return err;
			
	} else {
		return MESSAGE_NOT_THERE;
	}
}

int IMAPCopyMessage(int sock, const int uid, const string &tofolder, 
        const string &folderroot)
{
int err;
string line,command;

	if (tofolder == gIMAPINBOXName)
	{
		// form command
		command = gIMAPCopyTag + " " + gIMAPCopyCommand + " " + itoa(uid) + " " + tofolder + gIMAPEOL; 
	} else {
		// not INBOX
		// form command
		command = gIMAPCopyTag + " " + gIMAPCopyCommand + " " + itoa(uid) + " " + folderroot + gIMAPDirectoryIndicator + tofolder + gIMAPEOL; 
	}

		// send command
		write(sock, command.c_str() , strlen(command.c_str()));
	        if (gAlthea.get_Verbose()==2)
	          cout << "C: " << command << endl;	
		// read response

		line = readTilEOLTilExpected(sock,gIMAPCopyExpected,gIMAPCopyTag,err);
		if (err==LOST_CONNECTION_TO_SERVER) {return err;}
		if (strstr(line.c_str(),gIMAPCopySuccessful.c_str())!=NULL) {
			return err;		
		} else if (strstr(line.c_str(),gIMAPCopyFailedDueToNoFolder.c_str())!=NULL) { 
			return COPY_FAILED_DUE_TO_NO_FOLDER;
		} else if (strstr(line.c_str(),gIMAPCopyFailed.c_str())!=NULL){
			return COPY_FAILED;
		} else {
			return COPY_FAILED;
		}
}



int IMAPCheckMail(const int sock)
{

int err;
	string command = gIMAPCheckMailTag + " " + gIMAPCheckMailCommand + gIMAPEOL;
	// send command
	write(sock, command.c_str() , strlen(command.c_str()));
        if (gAlthea.get_Verbose()==2)
          cout << "C: " << command << endl;
	string line = readTilEOLTilExpected(sock,gIMAPCheckMailTag,gIMAPCheckMailExpected,err);

	return err;
}




// append posts the message to the folder


int IMAPAppend(int sock, const string &text, const string &folder, const string &folderroot)
{

  string line;

  int err;

  // form command
  string command = gIMAPAppendTag + " " + gIMAPAppendCommand + " " + folderroot +    "/" + folder + " (\\Seen) {"; 
  command = command + itoa(strlen(text.c_str())) + "}" + gIMAPEOL;
	// send command
        write(sock, command.c_str() , strlen(command.c_str()));
	if (gAlthea.get_Verbose()==2)
          cout << "C: " << command << endl;
	// read response
	write(sock, text.c_str(), strlen(text.c_str()));
	write(sock, gIMAPEOL.c_str(), strlen(gIMAPEOL.c_str()));

	line = readTilEOLTilExpected(sock,gIMAPAppendExpected,gIMAPAppendTag,err);
	if (err==LOST_CONNECTION_TO_SERVER) {return err;}

	if (strstr(gIMAPAppendSuccessful.c_str(),line.c_str())==0) 
	{
	  return err;
	}

	if (strstr(gIMAPAppendNoSuchFolder.c_str(),line.c_str())==0)
	{
	  return APPEND_FAILED_DUE_TO_NO_FOLDER;
	}
	
	if (strstr(gIMAPAppendFailed.c_str(),line.c_str())==0) 
	{
	  return APPEND_FAILED;
	}



return err;

}





