//
//  imap.h  --  IMAP functions 
//  -- created 5/23/00 updated 5/23/00
///////////////////////////////////////////

#ifndef IMAP_H
#define IMAP_H

#include        <iostream>
#include        <stdlib.h>
#include        <errno.h>
#include        <unistd.h>
#include        <string.h>
#include        <string>
#include        <list>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <netinet/in.h>
#include        "tcpUtilities.h"
#include        "basicNetwork.h"
#include        "imapLiterals.h"
#include        "errors.h"
#include        "Althea.h"

/////////////////////////////////////
// IMAP function prototypes.
/////////////////////////////////////

//login to the server

int IMAPLogin(int &sock, const string &host, const int port, const string &username, const string &password);

// get the folders in folderroot (eg of folderroot = "~/mail")

int getIMAPFolders(int sock, list<string>& folders, string folderroot);

// get the body of a message (not the header)

int IMAPGetMessageBodyByUID(int sock, int UID, string &messagetext);

// get the full header of a message

int IMAPGetMessageHeaderByUID(int sock, int UID, string &messagetext);

// sets (and unsets) message flags

int IMAPSetFlagByUID(int sock, int UID, const string &flagtoset, string &currentflags);
int IMAPUnSetFlagByUID(int sock, int UID, const string &flagtounset, string &currentflags);


// Expunge doze messages

int IMAPExpunge(int sock, list<int>& deleted);

// Disconnects you from the server

int IMAPLogout(int sock);

// make a folder commands
// to make a directory, the foldername should be followed by a "/"

int IMAPCreateFolder(int sock, const string &foldername, const string &folderroot);
int IMAPDeleteFolder(int sock, const string &foldername, const string &folderroot);
int IMAPRenameFolder(int sock, const string &fromname, const string &toname, const string &folderroot);

//
// copy a message
// to folder in folderroot.
//

int IMAPCopyMessage(int sock, const int uid, const string &tofolder, 
	const string &folderroot);

// convert sequence numbers to UIDs

int IMAPSeq2UID(const int sock, const int seq, int &UID);

// runs noop and returns the errorcode of getTilEOLTilExpected (so it will
// tell you when the server says BYE for example...

int IMAPCheckMail(const int sock);

// posts the text, text to the folder, folder

int IMAPAppend(const int sock, const string &text, const string &folder, const string &folderroot);




#endif




