//
//  load_config.cpp -- implementation of the functions to load the config file
//  -- created 4/20/00  updated 5/9/00
////////////////////////////////////////////////////////////

#include "load_config.h"

extern Althea gAlthea;

//  The master function that loads the configuration file into memory
int load_Althea()
{
	char c;
	string upper_string, name, smtp, imap, port, username, password, folder_root, fccfolder, field, filter_string, folder;
	char  *home_directory;
	ifstream local_config_in, rc_config_in;
	config_command command;
	int expert, cache;
	
	//  Get the environment variable HOME, this is POSIX and Windows Compliant
	home_directory = getenv ("HOME");
	strcat(home_directory, "/");
	strcat(home_directory, LOCAL_CONFIG_FILE); 
		
	local_config_in.open(home_directory);
	rc_config_in.open(RC_CONFIG_FILE);

	// If we do not have a config file in either /etc or home directory
	if ((local_config_in == NULL) && (rc_config_in == NULL) )
	{
		error ("No configuration file in home directory or /etc");
		return 1;
	}

	//  First load in rc configuration

	if (!(rc_config_in == NULL))
	{
	while (rc_config_in.get(c))
	{
		// read until you find an equal sign
		upper_string = *new string();
		if (c != '#')
		{
			if (islower(c))
				c = toupper(c);
			upper_string += c;
		
			while (rc_config_in.get(c))
			{
				if (c == '=')
					break;
				if (islower(c))
					c = toupper(c);
				upper_string += c;
			}

			if (rc_config_in == NULL)
			{
				error("Unexpected EOF in Config File");
				return 1;
			}
		
			command = convert_upper_to_command (upper_string);
			
			switch (command) 
			{
				case SERVER:
				  rc_config_in >> name >> smtp >> imap>> port >> username >> password >> folder_root >> fccfolder;
                            	        load_Server(name, smtp, imap, port, username, password, folder_root, fccfolder);
					if (gAlthea.get_Verbose())
                          	          cout << "server loaded" << endl;
                         	        break;
				case EXPERT:
					rc_config_in >> expert;
					load_Expert_Mode (expert);
					break;
		           	case FILTER:
				        rc_config_in >> field >> filter_string >> folder;
					Filter *F;
					F = new Filter( field, filter_string, folder );
					gAlthea.add_Filter( *F );
				        break;
				case CACHE:
					rc_config_in >> cache;
					load_Cache (cache);
					break;
				case NOCOMMAND:
					break;
			}
			
			// Make sure we are on the next command	line
			while (c != '\n') rc_config_in.get(c);	
		} 
	
		// We might have a comment, then we ignore the whole line.
		else if (c == '#')
		{
			// Ignore the line
			while (c != '\n') rc_config_in.get(c);
		}

		else 
		{
			cout << "Error:  File not in the proper format" << endl;
			return 2;
		}
	}  // end of big while 1
	rc_config_in.close();
	}  // end of big if

	//  Second, load in local configuration

	if (!(local_config_in == NULL))
	{
	while (local_config_in.get(c))
	{
		// read until you find an equal sign
		upper_string = *new string();
		if (c != '#')
		{
			if (islower(c))
				c = toupper(c);
			upper_string += c;
		
			while (local_config_in.get(c))
			{
				if (c == '=')
					break;
				if (islower(c))
					c = toupper(c);
				upper_string += c;
			}

			if (local_config_in == NULL)
			{
				error("Unexpected EOF in Config File");
				return 1;
			}
		
			command = convert_upper_to_command (upper_string);
			
			switch (command) 
			{
				case SERVER:
				  local_config_in >> name >> smtp >> imap>> port >> username >> password >> folder_root >> fccfolder;
                            	        load_Server(name, smtp, imap, port, username, password, folder_root, fccfolder);
					if (gAlthea.get_Verbose())
                          	          cout << "server loaded" << endl;
                         	        break;
				case EXPERT:
					local_config_in >> expert;
					load_Expert_Mode (expert);
					break;
			        case FILTER:
				        local_config_in >> field >> filter_string >> folder;
					Filter *F;
					F = new Filter( field, filter_string, folder );
					gAlthea.add_Filter( *F );
				        break;
				case CACHE:
					local_config_in >> cache;
					load_Cache (cache);
					break;
				case NOCOMMAND:
					break;
			}
			
			// Make sure we are on the next command	line
			while (c != '\n') local_config_in.get(c);	
		} // End of big if
	
		// We might have a comment, then we ignore the whole line.
		else if (c == '#')
		{
			// Ignore the line
			while (c != '\n') local_config_in.get(c);
		}

		else 
		{
			cout << "Error:  File not in the proper format" << endl;
			return 2;
		}
	}  // end of big while 2
	local_config_in.close();
	} // end of big if
	return 0;
}

// Other functions that will be used to load things into the data structure
void load_Server(const string & name, const string & smtp, const string & imap, const string & port, const string & username, const string & password, const string & folder_root, string & fccfolder )
{
	Server *S;

	int newport;

	newport = atoi(port.c_str());

	S = new Server( name, smtp, imap, newport, username, password, folder_root, fccfolder );
	gAlthea.add_Server( *S );
}

/*void load_Filter(const string & field, const string & filter_string, const string & folder)
{





}
*/

void load_Expert_Mode(int expert)
{
	Configuration temp_config = gAlthea.get_Configuration();
	if (expert)
		temp_config.set_expert_mode();
	else
		temp_config.set_beginner_mode();
}

config_command convert_upper_to_command(const string & upper_string)
{
	if (upper_string == "SERVER")
		return SERVER;
	if (upper_string == "EXPERT")
		return EXPERT;
	if (upper_string == "CACHE")
		return CACHE;
	if (upper_string == "FILTER")
	        return FILTER;
	return NOCOMMAND;
}
void load_Cache (int cache)
{
}













